﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;

namespace Fei.BaseLib {
    public class Seznam : ISeznam {
        private object syncRoot = new object();
        private void serad(ref SeznamUzel prvni, ref SeznamUzel posledni, int pocet) {
            if (pocet <= 1) {
                return;
            }
            SeznamUzel nasledujici;
            int polovina = pocet / 2;
            SeznamUzel stredni = prvni;
            for (int i = 1; i < polovina; i++) {
                stredni = stredni.Nasledujici;
            }
            nasledujici = stredni.Nasledujici;
            serad(ref prvni, ref stredni, polovina);
            serad(ref nasledujici, ref posledni, pocet - polovina);
            sluc(ref prvni, ref posledni, stredni);
        }
        private void sluc(ref SeznamUzel prvni, ref SeznamUzel posledni, SeznamUzel stredni) {
            SeznamUzel aktualni1 = prvni;
            SeznamUzel aktualni2 = stredni.Nasledujici;
            SeznamUzel nasledujici;
            while (aktualni1 != stredni.Nasledujici && aktualni2 != posledni.Nasledujici) {
                if ((aktualni1.Hodnota as IComparable).CompareTo(aktualni2.Hodnota) > 0) {
                    nasledujici = aktualni2.Nasledujici;
                    // vymazání uzlu aktuálního ze senzmau
                    if (aktualni2.Nasledujici != null) {
                        aktualni2.Nasledujici.Predchozi = aktualni2.Predchozi;
                    }
                    aktualni2.Predchozi.Nasledujici = aktualni2.Nasledujici;
                    if (aktualni2 == posledni) {
                        posledni = aktualni2.Predchozi;
                    }
                    // vložení uzlu aktuální2 před uzel aktuální1
                    if (aktualni1.Predchozi != null) {
                        aktualni1.Predchozi.Nasledujici = aktualni2;
                    }
                    aktualni2.Predchozi = aktualni1.Predchozi;
                    aktualni2.Nasledujici = aktualni1;
                    aktualni1.Predchozi = aktualni2;
                    if (aktualni1 == prvni) {
                        prvni = aktualni2;
                    }
                    aktualni2 = nasledujici;
                } else {
                    aktualni1 = aktualni1.Nasledujici;
                }
            }
        }
        public SeznamUzel Prvni { get; private set; }
        public SeznamUzel Posledni { get; private set; }
        public IEnumerable Uzly {
            get {
                SeznamUzel aktualni = this.Prvni;
                while (aktualni != null) {
                    yield return aktualni;
                }
            }
        }

        public virtual void PridejNaKonec(object prvek) {
            SeznamUzel uzel = new SeznamUzel(null, this.Posledni, prvek);
            if (this.Count == 0) {
                this.Prvni = uzel;
            } else {
                this.Posledni.Nasledujici = uzel;
            }
            this.Posledni = uzel;
            this.Count++;
        }

        public void Vymaz(SeznamUzel uzel) {
            if (uzel == Prvni) {
                Prvni = uzel.Nasledujici;
            } else {
                uzel.Predchozi.Nasledujici = uzel.Nasledujici;
            }
            if (uzel == Posledni) {
                Posledni = uzel.Predchozi;
            } else {
                uzel.Nasledujici.Predchozi = uzel.Predchozi;
            }

            /*
                        uzel.Nasledujici = null;
                        uzel.Predchozi = null;
                        uzel.Hodnota = null;
            */
            Count--;
        }

        public void Serad() {
            SeznamUzel prvni = Prvni, posledni = Posledni;
            serad(ref prvni, ref posledni, Count);
            Prvni = prvni;
            Posledni = posledni;

        }

        public virtual void PridejNaZacatek(object prvek) {
            SeznamUzel uzel = new SeznamUzel(this.Prvni, null, prvek);
            if (this.Count == 0) {
                this.Posledni = uzel;
            } else {
                this.Prvni.Predchozi = uzel;
            }
            this.Prvni = uzel;
            this.Count++;
        }

        // zde začínají regiony, který mi to nevygenerovalo...
        public bool Obsahuje(object prvek) {
            foreach (object item in this) {
                if(prvek.Equals(item)) {
                    return true;
                }
            }
        }

        void ISeznam.Pridej(object prvek) {
            this.PridejNaKonec(prvek);
        }

        public void Vymaz(object prvek) {
            foreach (SeznamUzel item in Uzly) {
                if (item.Hodnota.Equals(prvek)) {
                    Vymaz(item);
                    return;
                }
            }
            return;
        }

        public void VymazVse() {
            this.Prvni = null;
            this.Posledni = null;
            this.Count = 0;
            GC.Collect();
            GC.WaitForPendingFinalizers();
        }

        public void CopyTo(Array array, int index) {
            throw new NotImplementedException();
        }

        public int Count { get; private set; } // automaticky implementovaná vlastnost

        public bool IsSynchronized {
            get { return false; }
        }

        public object SyncRoot {
            get { return this.syncRoot; }
        }

        public System.Collections.IEnumerator GetEnumerator() {
            foreach (SeznamUzel item in Uzly) {
                yield return item.Hodnota;
            }
        }
    }
}
